@@ -1,4 +1,4 @@
-//! ## Snapshot testing for a herd of CLI tests
+//! # Snapshot testing for a herd of CLI tests
 //!
 //! `trycmd` is a test harness that will enumerate test case files and run them to verify the
 //! results, taking inspiration from
@@ -13,7 +13,7 @@
 //!     with [mdbook](https://rust-lang.github.io/mdBook/)
 //! - [cram](https://bitheap.org/cram/): For cases agnostic of any programming language
 //!
-//! ### Getting Started
+//! ## Getting Started
 //!
 //! To create a minimal setup, create a `tests/cli_tests.rs` with
 //! ```rust,no_run
@@ -38,62 +38,7 @@
 //! }
 //! ```
 //!
-//! ### File Formats
-//!
-//! Say you have `tests/cmd/help.trycmd` (or `help.toml`), `trycmd` will look for:
-//! - `tests/cmd/help.stdin`
-//! - `tests/cmd/help.stdout`
-//! - `tests/cmd/help.stderr`
-//!
-//! #### `*.trycmd`
-//!
-//! A `trycmd` file is just a command with arguments, with the arguments split with [shlex](https://crates.io/crates/shlex).
-//!
-//! The command is interpreted as `bin.name` in a `toml` file.
-//!
-//! #### `*.toml`
-//!
-//! As an alternative to `.trycmd`, he `toml` files give you a lot more control over how your command runs.
-//!
-//! [schema](https://github.com/assert-rs/trycmd/blob/main/schema.json):
-//! - `bin.name`: The name of the binary target from `Cargo.toml` to be used to find the file path
-//!
-//! #### `*.stdin`
-//!
-//! Data to pass to `stdin`.
-//! - If not present, nothing will be written to `stdin`
-//! - If `binary = false` in `*.toml` (the default), newlines will be normalized.
-//!
-//! #### `*.stdout` and `*.stderr`
-//!
-//! Expected results for `stdout` or `stderr`.
-//! - If not present, we'll not verify the output
-//! - If `binary = false` in `*.toml` (the default), newlines will be normalized before comparing
-//!
-//! ##### Eliding Content
-//!
-//! Sometimes the output either includes:
-//! - Content that changes from run-to-run (like time)
-//! - Content out of scope of your tests and you want to exclude it to reduce brittleness
-//!
-//! To elide a section of content:
-//! - `...` as its own line will match all lines until the next one.  This is equivalent of
-//!   `(([^\n]*\n)*?`.
-//!
-//! We will preserve these with `TRYCMD=dump` and will make a best-effort at preserving them with
-//! `TRYCMD=overwrite`.
-//!
-//! #### `*.in/`
-//!
-//! When present, this will automatically be picked as the CWD for the command
-//!
-//! #### `*.out/`
-//!
-//! When present, each file in this directory will be compared to generated or modified files.
-//!
-//! See also "Eliding Content" for `.stdout`
-//!
-//! ### Workflow
+//! ## Workflow
 //!
 //! To generate snapshots, run
 //! ```bash
@@ -114,19 +59,123 @@
 //! cargo test --test cli_tests -- cli_tests trycmd=name1 trycmd=name2...
 //! ```
 //! To filter the tests to those with `name1`, `name2`, etc in their file names.
+//!
+//! ## File Formats
+//!
+//! Say you have `tests/cmd/help.trycmd`, `trycmd` will look for:
+//! - `tests/cmd/help.in/`
+//! - `tests/cmd/help.out/`
+//!
+//! For `tests/cmd/help.toml`, `trycmd` will look for:
+//! - `tests/cmd/help.stdin`
+//! - `tests/cmd/help.stdout`
+//! - `tests/cmd/help.stderr`
+//! - `tests/cmd/help.in/`
+//! - `tests/cmd/help.out/`
+//!
+//! ### `*.trycmd`
+//!
+//! `*.trycmd` files are literate test cases good for:
+//! - Markdown-compatible syntax for directly rendering them
+//! - Terminal-like appearance for extracting subsections into documentation
+//! - Reducing the proliferation of files
+//! - Running multiple commands within the same temp dir
+//!
+//! The syntax is:
+//! - Test cases live inside of ` ``` ` fenced code blocks
+//!   - Everything out of them is ignored
+//!   - Blocks with info strings with an unsupported language (not `trycmd`, `bash`, `sh`) or the
+//!     `ignore` attribute are ignored
+//! - "`$ `" line prefix starts a new command
+//! - "`> `" line prefix appends to the prior command
+//! - "`? <status>`" line indicates the exit code (like `echo "? $?"`) and `<status>` can be
+//!   - An exit code
+//!   - `success` *(default)*, `failed`, `interrupted`, `skipped`
+//!  - All following lines are treated as stdout + stderr
+//!
+//! The command is then split with [shlex](https://crates.io/crates/shlex), allowing quoted content
+//! to allow spaces.  The first argument is the program to run which maps to `bin.name` in the
+//! `.toml` file.
+//!
+//! Example:
+//! ~~~md
+//! With the following code:
+//! ```rust
+//! println!("{}", message);
+//! ```
+//!
+//! You get the following:
+//! ```
+//! $ my-cmd --print 'Hello World'
+//! Hello
+//! ```
+//! ~~~
+//!
+//! ### `*.toml`
+//!
+//! As an alternative to `.trycmd`, the `toml` are good for:
+//! - Precise control over current dir, stdin/stdout/stderr (including binary support)
+//! - 1-to-1 with dumped results
+//! - `TRYCMD=overwrite` support
+//!
+//! [schema](https://github.com/assert-rs/trycmd/blob/main/schema.json):
+//! - `bin.name`: The name of the binary target from `Cargo.toml` to be used to find the file path
+//!
+//! #### `*.stdin`
+//!
+//! Data to pass to `stdin`.
+//! - If not present, nothing will be written to `stdin`
+//! - If `binary = false` in `*.toml` (the default), newlines will be normalized.
+//!
+//! #### `*.stdout` and `*.stderr`
+//!
+//! Expected results for `stdout` or `stderr`.
+//! - If not present, we'll not verify the output
+//! - If `binary = false` in `*.toml` (the default), newlines will be normalized before comparing
+//!
+//! **Eliding Content**
+//!
+//! Sometimes the output either includes:
+//! - Content that changes from run-to-run (like time)
+//! - Content out of scope of your tests and you want to exclude it to reduce brittleness
+//!
+//! To elide a section of content:
+//! - `...` as its own line: match all lines until the next one.  This is equivalent of
+//!   `\n(([^\n]*\n)*?`.
+//!
+//! We will preserve these with `TRYCMD=dump` and will make a best-effort at preserving them with
+//! `TRYCMD=overwrite`.
+//!
+//! ### `*.in/`
+//!
+//! When present, this will automatically be picked as the CWD for the command.
+//!
+//! `.keep` files will be ignored but their parent directories will be created.
+//!
+//! ### `*.out/`
+//!
+//! When present, each file in this directory will be compared to generated or modified files.
+//!
+//! See also "Eliding Content" for `.stdout`
+//!
+//! `.keep` files will be ignored.
+
+// Doesn't distinguish between incidental sharing vs essential sharing
+#![allow(clippy::branches_sharing_code)]
 
 pub mod cargo;
 pub mod schema;
 
-mod cases;
-mod color;
-mod command;
 #[cfg(feature = "diff")]
 pub(crate) mod diff;
 pub(crate) mod elide;
+pub(crate) mod lines;
+
+mod cases;
+mod color;
+mod command;
 mod error;
 mod filesystem;
-pub(crate) mod lines;
 mod registry;
 mod runner;
 mod spec;
@@ -136,8 +185,7 @@ pub use error::Error;
 
 pub(crate) use color::Palette;
 pub(crate) use command::wait_with_input_output;
-pub(crate) use filesystem::{shallow_copy, FilesystemContext, Iterate as FsIterate};
+pub(crate) use filesystem::{shallow_copy, File, FilesystemContext, Iterate as FsIterate};
 pub(crate) use registry::BinRegistry;
 pub(crate) use runner::{Case, Mode, Runner};
-pub(crate) use schema::{Bin, CommandStatus, Env, TryCmd};
 pub(crate) use spec::RunnerSpec;
